// Utility functions
const $ = (selector) => document.querySelector(selector);
const $$ = (selector) => document.querySelectorAll(selector);

// DOM Elements Cache
const DOM = {
    body: document.body,
    root: document.documentElement,
    profileBtn: $('#profileBtn'),
    profileDropdown: $('#profileDropdown'),
    botDropdown: $('#botDropdown'),
    themeToggle: $('#themeToggle'),
    logoutBtn: $('#logoutBtn'),
    coinBalance: $('#coinBalance'),
    mainContent: $('.main')
};

// Theme Management
const ThemeManager = {
    init() {
        const savedTheme = localStorage.getItem('theme') || 'light';
        this.setTheme(savedTheme);
        DOM.themeToggle.addEventListener('click', () => {
            const currentTheme = DOM.root.getAttribute('data-theme');
            this.setTheme(currentTheme === 'dark' ? 'light' : 'dark');
        });
    },

    setTheme(theme) {
        DOM.root.setAttribute('data-theme', theme);
        localStorage.setItem('theme', theme);
        DOM.themeToggle.innerHTML = theme === 'dark'
            ? '<i class="fas fa-sun"></i>'
            : '<i class="fas fa-moon"></i>';
    }
};

// Dropdown Management
const DropdownManager = {
    init() {
        // Handle profile dropdown
        DOM.profileBtn?.addEventListener('click', (e) => {
            e.stopPropagation();
            DOM.profileDropdown?.classList.toggle('show');
            DOM.botDropdown?.classList.remove('show');
        });

        // Close dropdowns when clicking outside
        document.addEventListener('click', () => {
            DOM.profileDropdown?.classList.remove('show');
        });

        // Prevent dropdown from closing when clicking inside
        DOM.profileDropdown?.addEventListener('click', (e) => {
            e.stopPropagation();
        });
    }
};

// Loading Screen
const LoadingManager = {
    init() {
        const loadingScreen = document.createElement('div');
        const loadingText = document.createElement('div');
        loadingScreen.id = 'loading-screen';
        loadingText.id = 'loading-text';
        DOM.body.appendChild(loadingScreen);
        loadingScreen.appendChild(loadingText);

        const messages = ['Loading', 'Loading.', 'Loading..', 'Loading...'];
        let currentIndex = 0;
        const interval = setInterval(() => {
            loadingText.textContent = messages[currentIndex];
            currentIndex = (currentIndex + 1) % messages.length;
        }, 200);

        setTimeout(() => {
            clearInterval(interval);
            loadingScreen.style.display = 'none';
        }, 1500);
    }
};

// Network Status Manager
const NetworkManager = {
    init() {
        this.createOfflineOverlay();
        this.setupEventListeners();
        this.checkNetworkStatus();
    },

    createOfflineOverlay() {
        const overlay = document.createElement('div');
        overlay.id = 'offlineOverlay';
        overlay.className = 'offline-overlay';
        overlay.innerHTML = `
            <div class="offline-popup">
                <i class="fas fa-wifi fa-3x mb-4" style="color: var(--neutral); opacity: 0.5;"></i>
                <h3>Oops! You seem to be offline.</h3>
                <p>Please check your network connection and try again.</p>
            </div>
        `;
        DOM.body.appendChild(overlay);
    },

    setupEventListeners() {
        window.addEventListener('online', () => this.checkNetworkStatus());
        window.addEventListener('offline', () => this.checkNetworkStatus());
    },

    checkNetworkStatus() {
        const offlineOverlay = $('#offlineOverlay');
        if (navigator.onLine) {
            DOM.mainContent?.classList.remove('blurred');
            offlineOverlay.style.opacity = '0';
            offlineOverlay.style.visibility = 'hidden';
        } else {
            DOM.mainContent?.classList.add('blurred');
            offlineOverlay.style.opacity = '1';
            offlineOverlay.style.visibility = 'visible';
        }
    }
};

// Ad Block Detector
const AdBlockDetector = {
    isOverlayShown: false,
    detectionInterval: null,

    init() {
        this.createOverlay();
        setTimeout(() => this.startContinuousDetection(), 2000);
    },

    createOverlay() {
        if ($('#adblock-overlay')) return;
        
        const overlay = document.createElement('div');
        overlay.id = 'adblock-overlay';
        overlay.style.display = 'none';
        overlay.innerHTML = `
            <div class="adblock-warning">
                <i class="fas fa-exclamation-circle fa-3x mb-4"></i>
                <h2>AdBlock Detected</h2>
                <p>We noticed you're using an ad blocker. Our site relies on advertising revenue to provide free content.</p>
                <p>Please disable your ad blocker to continue using our site.</p>
                <button id="adblock-retry" class="btn btn-primary mt-4">
                    I've disabled my ad blocker
                </button>
            </div>
        `;
        DOM.body.appendChild(overlay);

        overlay.querySelector('#adblock-retry').addEventListener('click', () => {
            location.reload(); // Refresh the page when button is clicked
        });
    },

    startContinuousDetection() {
        if (this.detectionInterval) {
            clearInterval(this.detectionInterval);
        }
        this.checkForAdBlocker();
        this.detectionInterval = setInterval(() => this.checkForAdBlocker(), 2000);
    },

    checkForAdBlocker() {
        const testAd = document.createElement('div');
        testAd.className = 'ads ad-banner-example pub_300x250';
        testAd.style.cssText = 'position: absolute; top: -999px; width: 1px; height: 1px;';
        DOM.body.appendChild(testAd);

        const adSenseScript = document.querySelector('script[src*="adsbygoogle.js"]');
        
        setTimeout(() => {
            const isBlocked = (
                testAd.offsetHeight === 0 ||
                !adSenseScript ||
                testAd.clientHeight === 0
            );

            if (isBlocked) {
                this.onDetected();
            } else {
                this.hideAdBlockWarning();
            }
            testAd.remove();
        }, 100);
    },

    onDetected() {
        if (!this.isOverlayShown) {
            this.isOverlayShown = true;
            this.showAdBlockWarning();
        }
    },

    showAdBlockWarning() {
        const overlay = $('#adblock-overlay');
        if (overlay && DOM.mainContent) {
            overlay.style.display = 'flex';
            DOM.mainContent.style.filter = 'blur(5px)';
            DOM.body.style.overflow = 'hidden';
            
            overlay.style.position = 'fixed';
            overlay.style.top = '0';
            overlay.style.left = '0';
            overlay.style.width = '100%';
            overlay.style.height = '100%';
            overlay.style.backgroundColor = 'rgba(0, 0, 0, 0.8)';
            overlay.style.zIndex = '9999';
        }
    },

    hideAdBlockWarning() {
        const overlay = $('#adblock-overlay');
        if (overlay && DOM.mainContent) {
            overlay.style.display = 'none';
            DOM.mainContent.style.filter = 'none';
            DOM.body.style.overflow = 'auto';
        }
        this.isOverlayShown = false;
    }
};

// Ad Manager
const AdManager = {
    adUnits: {
        main: { slot: "6493686389", positions: ['top', 'sidebar', 'content-middle'] },
        bottom: { slot: "3527508043", positions: ['footer'] }
    },
    initialized: false,
    createdAds: new Set(),

    init() {
        if (this.initialized) return;
        this.initialized = true;

        this.insertAdSenseScript()
            .then(() => {
                this.createMainAds();
                this.createBottomAd();
                this.setupErrorHandling();
            })
            .catch(error => {
                console.warn('Error initializing AdSense:', error);
                this.handleAdError();
            });
    },

    insertAdSenseScript() {
        return new Promise((resolve, reject) => {
            if (document.querySelector('script[src*="adsbygoogle.js"]')) {
                resolve();
                return;
            }

            const script = document.createElement('script');
            script.src = "https://pagead2.googlesyndication.com/pagead/js/adsbygoogle.js?client=ca-pub-5279011797376172";
            script.async = true;
            script.crossOrigin = "anonymous";
            script.onload = resolve;
            script.onerror = reject;
            document.head.appendChild(script);
        });
    },

    setupErrorHandling() {
        window.onerror = (msg, src, lineNo, colNo, error) => {
            if (msg.includes('adsbygoogle') || src?.includes('adsbygoogle')) {
                this.handleAdError();
                return true;
            }
            return false;
        };

        window.addEventListener('error', (e) => {
            if (e.target?.src?.includes('adsbygoogle')) {
                this.handleAdError();
            }
        }, true);
    },

    handleAdError() {
        console.warn('AdSense error detected');
        setTimeout(() => this.reinitializeAds(), 5000);
    },

    reinitializeAds() {
        this.createdAds.clear();
        $$('.ad-container').forEach(container => container.remove());
        this.createMainAds();
        this.createBottomAd();
    },

    createMainAds() {
        this.adUnits.main.positions.forEach(position => {
            if (!this.createdAds.has(`main-${position}`)) {
                this.createAdContainer(position, this.adUnits.main.slot);
                this.createdAds.add(`main-${position}`);
            }
        });
    },

    createBottomAd() {
        if (this.createdAds.has('bottom')) return;

        const bottomContainer = document.createElement('div');
        bottomContainer.className = 'ad-container ad-bottom';
        bottomContainer.style.margin = '20px 0';
        
        const adId = `ad-bottom-${Date.now()}`;
        bottomContainer.innerHTML = this.createAdHTML(adId, this.adUnits.bottom.slot);
        
        DOM.body.appendChild(bottomContainer);
        this.createdAds.add('bottom');
        this.pushAd(adId);
    },

    createAdContainer(position, adSlot) {
        const container = document.createElement('div');
        container.className = `ad-container ad-${position}`;
        
        const adId = `ad-${position}-${Date.now()}`;
        container.innerHTML = this.createAdHTML(adId, adSlot);

        this.insertAtPosition(container, position);
        this.pushAd(adId);
    },

    createAdHTML(adId, adSlot) {
        return `
            <ins class="adsbygoogle"
                 id="${adId}"
                 style="display:block"
                 data-ad-client="ca-pub-5279011797376172"
                 data-ad-slot="${adSlot}"
                 data-ad-format="auto"
                 data-full-width-responsive="true"></ins>
        `;
    },

    insertAtPosition(container, position) {
        const main = $('.main');
        const positions = {
            'top': () => main?.insertBefore(container, main.firstChild),
            'sidebar': () => $('.sidebar')?.appendChild(container),
            'content-middle': () => {
                const content = $('.content');
                if (content) {
                    const midPoint = Math.floor(content.children.length / 2);
                    content.insertBefore(container, content.children[midPoint]);
                }
            },
            'footer': () => main?.appendChild(container)
        };

        if (positions[position] && !$(`.ad-container.ad-${position}`)) {
            positions[position]();
        }
    },

    pushAd(adId) {
        setTimeout(() => {
            const adElement = document.getElementById(adId);
            if (adElement && !adElement.dataset.adInitialized) {
                (window.adsbygoogle = window.adsbygoogle || []).push({});
                adElement.dataset.adInitialized = 'true';
            }
        }, 100);
    }
};

// Snow Effect
const SnowEffect = {
    init() {
        this.createToggleButton();
        const snowEnabled = localStorage.getItem('snowEffect') === 'true';
        this.updateSnowEffect(snowEnabled);
    },

    createToggleButton() {
        const navControls = $('.nav-controls');
        const snowToggle = document.createElement('button');
        snowToggle.id = 'snowToggle';
        snowToggle.className = 'btn btn-secondary';
        snowToggle.innerHTML = '<i class="fas fa-snowflake"></i>';
        
        const themeToggle = $('#themeToggle');
        navControls?.insertBefore(snowToggle, themeToggle);

        snowToggle.addEventListener('click', () => {
            const isSnowing = !DOM.body.classList.contains('snowing');
            this.updateSnowEffect(isSnowing);
        });
    },

    createSnowContainer() {
        if (!$('.snow-container')) {
            const container = document.createElement('div');
            container.classList.add('snow-container');
            DOM.body.appendChild(container);
        }
        return $('.snow-container');
    },

    createSnowflake() {
        const snowflake = document.createElement('div');
        snowflake.classList.add('snowflake');
        snowflake.innerHTML = '❄';
        
        const topOffset = Math.random() * 100 - 100;
        snowflake.style.top = `${topOffset}px`;
        snowflake.style.left = `${Math.random() * 100}vw`;
        
        const size = Math.random() * 8 + 8;
        snowflake.style.fontSize = `${size}px`;
        
        const duration = Math.random() * 4 + 4;
        snowflake.style.animationDuration = `${duration}s`;
        
        const wobble = Math.random() * 50 - 25;
        snowflake.style.transform = `translateX(${wobble}px)`;
        snowflake.style.animationTimingFunction = `cubic-bezier(${Math.random()}, ${Math.random()}, ${Math.random()}, ${Math.random()})`;
        
        const container = this.createSnowContainer();
        container.appendChild(snowflake);
        
        setTimeout(() => snowflake.remove(), duration * 1000);
    },

   startSnow() {
        for(let i = 0; i < 50; i++) {
            setTimeout(() => this.createSnowflake(), Math.random() * 2000);
        }
        
        setInterval(() => {
            for(let i = 0; i < 3; i++) {
                this.createSnowflake();
            }
        }, 100);
    },

    stopSnow() {
        const container = $('.snow-container');
        container?.remove();
    },

    updateSnowEffect(enabled) {
        const snowToggle = $('#snowToggle');
        
        if (enabled) {
            DOM.body.classList.add('snowing');
            this.startSnow();
            snowToggle.style.opacity = '1';
        } else {
            DOM.body.classList.remove('snowing');
            this.stopSnow();
            snowToggle.style.opacity = '0.5';
        }
        
        localStorage.setItem('snowEffect', enabled);
    }
};

// Profile Management
const ProfileManager = {
    async loadProfilePicture() {
        const profilePic = $('#profileBtn');
        const placeholder = $('#profilePlaceholder');
    
        try {
            const response = await fetch('/get-current-user');
            const data = await response.json();
    
            if (data.success && data.profile_picture) {
                const img = new Image();
                img.src = data.profile_picture;
                
                img.onload = () => {
                    if (profilePic) {
                        profilePic.src = data.profile_picture;
                        profilePic.style.display = 'block';
                    }
                };
            }
        } catch (error) {
            console.error('Error loading profile picture:', error);
        }
    }
};

// Authentication Management
const AuthManager = {
    async checkLogin() {
        try {
            const response = await fetch('/check-login');
            
            if (!response.ok) {
                window.location.href = '/auth/login';
                return;
            }

            const session = await response.json();

            if (session.user?.is_banned === 1) {
                window.location.href = '/banned';
                return;
            }

            if (!session.user) {
                window.location.href = '/auth/login';
            }
        } catch (error) {
            console.error('Authentication error:', error);
            this.showError('Authentication error. Please try again later.');
        }
    },

    setupLogout() {
        DOM.logoutBtn?.addEventListener('click', async () => {
            try {
                const response = await fetch('/logout');
                if (response.ok) {
                    window.location.href = '/auth/login';
                } else {
                    throw new Error('Logout failed');
                }
            } catch (error) {
                this.showError('Logout failed');
            }
        });
    },

    showError(message) {
        // Implement your error notification system here
        console.error(message);
    }
};

// Coin Management
const CoinManager = {
    async updateCoinBalance() {
        try {
            const response = await fetch('/user-coins');
            const data = await response.json();
            if (DOM.coinBalance) {
                DOM.coinBalance.textContent = `${data.coins} Coins`;
            }
        } catch (error) {
            console.error('Error fetching coin balance:', error);
        }
    }
};

// Analytics Setup
const AnalyticsManager = {
    init() {
        this.setupGoogleAnalytics();
        this.setupGoogleAds();
    },

    setupGoogleAnalytics() {
        const gtagScript = document.createElement('script');
        gtagScript.async = true;
        gtagScript.src = "https://www.googletagmanager.com/gtag/js?id=G-ZV0YB41291";

        const inlineScript = document.createElement('script');
        inlineScript.textContent = `
            window.dataLayer = window.dataLayer || [];
            function gtag(){dataLayer.push(arguments);}
            gtag('js', new Date());
            gtag('config', 'G-ZV0YB41291');
        `;

        document.head.appendChild(gtagScript);
        document.head.appendChild(inlineScript);
    },

    setupGoogleAds() {
        const script = document.createElement('script');
        script.src = "https://pagead2.googlesyndication.com/pagead/js/adsbygoogle.js?client=ca-pub-5279011797376172";
        script.async = true;
        script.crossOrigin = "anonymous";
        document.head.appendChild(script);
    }
};


// Notification System Implementation
const NotificationSystem = {
    init() {
        this.createNotificationBell();
        this.initializeEventListeners();
        this.startNotificationCheck();
    },

    createNotificationBell() {
        const navControls = $('.nav-controls');
        const bellButton = document.createElement('div');
        bellButton.className = 'notification-bell';
        bellButton.innerHTML = `
            <button class="btn btn-secondary" id="notificationBtn">
                <i class="fas fa-bell"></i>
                <span class="notification-badge" id="notificationBadge">0</span>
            </button>
            <div class="notification-dropdown" id="notificationDropdown">
                <div class="notification-header">
                    <h3>Notifications</h3>
                    <button class="mark-all-read" id="markAllRead">
                        Mark all as read
                    </button>
                </div>
                <div class="notification-list" id="notificationList">
                    <!-- Notifications will be inserted here -->
                </div>
            </div>
        `;
        
        // Insert bell before the theme toggle button
        const themeToggle = $('#themeToggle');
        navControls?.insertBefore(bellButton, themeToggle);
    },

    initializeEventListeners() {
        const notificationBtn = $('#notificationBtn');
        const notificationDropdown = $('#notificationDropdown');
        const markAllRead = $('#markAllRead');

        notificationBtn?.addEventListener('click', (e) => {
            e.stopPropagation();
            notificationDropdown?.classList.toggle('show');
            if (notificationDropdown?.classList.contains('show')) {
                this.markNotificationsAsSeen();
            }
        });

        document.addEventListener('click', (e) => {
            if (notificationDropdown && 
                !notificationDropdown.contains(e.target) && 
                notificationBtn && 
                !notificationBtn.contains(e.target)) {
                notificationDropdown.classList.remove('show');
            }
        });

        markAllRead?.addEventListener('click', async () => {
            await this.markAllNotificationsAsRead();
        });
    },

    async fetchNotifications() {
        try {
            const response = await fetch('/api/notifications');
            const data = await response.json();
            
            if (response.ok) {
                this.updateNotificationUI(data.notifications);
                this.updateBadgeCount(data.unreadCount);
            }
        } catch (error) {
            console.error('Error fetching notifications:', error);
        }
    },

    updateNotificationUI(notifications) {
        const notificationList = $('#notificationList');
        if (!notificationList) return;
        
        if (!notifications?.length) {
            notificationList.innerHTML = `
                <div class="no-notifications">
                    <p>No notifications yet</p>
                </div>
            `;
            return;
        }

        notificationList.innerHTML = notifications.map(notification => `
            <div class="notification-item ${notification.read ? 'read' : 'unread'} ${notification.link ? 'clickable' : ''}" 
                 data-notification-id="${notification.id}"
                 ${notification.link ? `onclick="NotificationSystem.handleNotificationClick(event, ${notification.id}, '${notification.link}')"` : ''}>
                <div class="notification-content">
                    <p>${notification.message}</p>
                    <span class="notification-time">
                        ${this.formatTime(notification.created_at)}
                    </span>
                </div>
                ${!notification.read ? `
                    <button class="mark-read-btn" onclick="NotificationSystem.handleMarkAsRead(event, ${notification.id})">
                        <i class="fas fa-check"></i>
                    </button>
                ` : ''}
            </div>
        `).join('');
    },

    handleNotificationClick(event, notificationId, link) {
        event.preventDefault();
        this.markAsRead(notificationId)
            .then(() => {
                window.location.href = link;
            })
            .catch(error => {
                console.error('Error handling notification click:', error);
            });
    },

    handleMarkAsRead(event, notificationId) {
        event.stopPropagation();
        this.markAsRead(notificationId);
    },

    async markAsRead(notificationId) {
        try {
            const response = await fetch(`/api/notifications/${notificationId}/read`, {
                method: 'POST'
            });
            
            if (response.ok) {
                await this.fetchNotifications();
            }
        } catch (error) {
            console.error('Error marking notification as read:', error);
        }
    },

    async markAllNotificationsAsRead() {
        try {
            const response = await fetch('/api/notifications/mark-all-read', {
                method: 'POST'
            });
            
            if (response.ok) {
                await this.fetchNotifications();
            }
        } catch (error) {
            console.error('Error marking all notifications as read:', error);
        }
    },

    async markNotificationsAsSeen() {
        try {
            const response = await fetch('/api/notifications/mark-seen', {
                method: 'POST'
            });
            
            if (response.ok) {
                await this.fetchNotifications();
            }
        } catch (error) {
            console.error('Error marking notifications as seen:', error);
        }
    },

    updateBadgeCount(count) {
        const badge = $('#notificationBadge');
        if (badge) {
            badge.textContent = count;
            badge.style.display = count > 0 ? 'block' : 'none';
        }
    },

    formatTime(timestamp) {
        const date = new Date(timestamp);
        const now = new Date();
        const diff = now - date;
        
        // Less than 24 hours
        if (diff < 86400000) {
            return date.toLocaleTimeString([], { hour: '2-digit', minute: '2-digit' });
        }
        // Less than 7 days
        if (diff < 604800000) {
            return date.toLocaleDateString([], { weekday: 'short' });
        }
        // Otherwise
        return date.toLocaleDateString([], { month: 'short', day: 'numeric' });
    },

    startNotificationCheck() {
        this.fetchNotifications();
        setInterval(() => this.fetchNotifications(), 30000);
    }
};

// Make sure to add NotificationSystem to the DOMContentLoaded initialization
document.addEventListener('DOMContentLoaded', () => {
    // ... other initializations ...
    NotificationSystem.init();
});
// Initialize everything when DOM is ready
document.addEventListener('DOMContentLoaded', () => {
    // Initialize managers
    ThemeManager.init();
    DropdownManager.init();
    LoadingManager.init();
    NetworkManager.init();
    AdBlockDetector.init();
    AdManager.init();
    SnowEffect.init();
    AnalyticsManager.init();

    // Setup authentication and profile
    AuthManager.setupLogout();
    ProfileManager.loadProfilePicture();
    
    // Start periodic checks
    setInterval(() => AuthManager.checkLogin(), 60000);
    CoinManager.updateCoinBalance();
});

// Export modules if needed
window.AdManager = AdManager;
window.NotificationSystem = NotificationSystem; // If you're using the notification system